{ --  TDigitsPanel --

  Feel free to use it ! But please, don't forget to send me yours comments ...

  Author : Benoit MICHAUD
  E-Mail : BMichaud@mnet.fr

  History      1.00 - October 18th, 1997
                        Some corrections : Thanks to Michal Bukovjan
                        - Event "DigitsChanged" implemented.
                        - Property "Color" removed, use now the "transparent color".
               0.90 - July 25th, 1997
                        First public release.
}

unit DigitsPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TDigitsLength = 1..8;

  TDigitsPanel = class(TGraphicControl)
  private
    FDigits: TBitMap;
    FDigitWidth, FDigitHeight: Word;
    FBevelOuter: TPanelBevel;
    FBevelInner: TPanelBevel;
    FBevelWidth: Byte;
    FColor: TColor;
    FLength: TDigitsLength;
    FMargin: Byte;
    FValue: Longint;
  protected
    procedure SetDigits(Value: TBitMap);
    procedure SetBevelOuter(Value: TPanelBevel);
    procedure SetBevelInner(Value: TPanelBevel);
    procedure SetBevelWidth(Value: Byte);
    procedure SetColor(Value: TColor);
    procedure SetLength(Value: TDigitsLength);
    procedure SetMargin(Value: Byte);
    procedure SetValue(Value: Longint);
    procedure DigitsChanged(Sender: TObject);
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Digits: TBitMap read FDigits write SetDigits;
    property BevelOuter: TPanelBevel read FBevelOuter write SetBevelOuter default bvRaised;
    property BevelInner: TPanelBevel read FBevelInner write SetBevelInner default bvLowered;
    property BevelWidth: Byte read FBevelWidth write SetBevelWidth default 2;
    property Color: TColor read FColor write SetColor default clBlack;
    property Length: TDigitsLength read FLength write SetLength default 4;
    property Margin: Byte read FMargin write SetMargin default 2;
    property Value: Longint read FValue write SetValue default 0;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TDigitsPanel]);
end;

{ -- TDigitsPanel - Constructor & Destructor -- }

constructor TDigitsPanel.Create(AOwner: TComponent);
begin
  // TGraphicControl
  inherited Create(AOwner);
  ControlStyle := ControlStyle+[csOpaque,csReplicatable];  {???}
  Width := 100; Height := 20;
  // TDigitsPanel
  FDigits := TBitMap.Create;
  FDigits.OnChange := DigitsChanged;
  FBevelOuter := bvRaised; FBevelInner := bvLowered; FBevelWidth := 2;
  FColor := clBlack;
  FLength := 4;
  FMargin := 2;
  FValue := 0;
end;

destructor TDigitsPanel.Destroy;
begin
  FDigits.Free;
  inherited Destroy;
end;

{ -- TDigitsPanel - Properties -- }

procedure TDigitsPanel.SetDigits(Value: TBitMap);
begin
 if Value<>nil then FDigits.Assign(Value);
end;

procedure TDigitsPanel.SetBevelOuter(Value: TPanelBevel);
begin
  FBevelOuter := Value;
  SetBounds(Left,Top,0,0);
  Invalidate;
end;

procedure TDigitsPanel.SetBevelInner(Value: TPanelBevel);
begin
  FBevelInner := Value;
  SetBounds(Left,Top,0,0);
  Invalidate;
end;

procedure TDigitsPanel.SetBevelWidth(Value: Byte);
begin
  FBevelWidth := Value;
  SetBounds(Left,Top,0,0);
end;

procedure TDigitsPanel.SetColor(Value: TColor);
begin
  FColor := Value;
  Invalidate;
end;

procedure TDigitsPanel.SetLength(Value: TDigitsLength);
begin
  FLength := Value;
  SetBounds(Left,Top,0,0);
end;

procedure TDigitsPanel.SetMargin(Value: Byte);
begin
  FMargin := Value;
  SetBounds(Left,Top,0,0);
end;

procedure TDigitsPanel.SetValue(Value: Longint);
begin
  FValue := Abs(Value) mod Trunc(Exp(FLength*Ln(10)));
  Invalidate;
end;

{ -- TDigitsPanel - Procedures -- }

procedure TDigitsPanel.DigitsChanged(Sender: TObject);
begin
  FDigitWidth := FDigits.Width div 11;
  FDigitHeight := FDigits.Height;
  FColor := FDigits.TransparentColor;
  SetBounds(Left,Top,0,0);
end;

procedure TDigitsPanel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  Factor: Integer;
begin
  Factor := Integer(BevelOuter<>bvNone)+Integer(BevelInner<>bvNone);
  inherited SetBounds(ALeft,
                      ATop,
                      FBevelWidth*Factor*2+FMargin+FLength*(FDigitWidth+FMargin),
                      FBevelWidth*Factor*2+FMargin+(FDigitHeight+FMargin));
end;

procedure TDigitsPanel.Paint;
var
  TmpRect: TRect;
  TopColor, BottomColor: TColor;
  ValueString: String;
  Digit: Integer;
  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;
begin
  TmpRect := ClientRect;
  if BevelOuter<>bvNone then
    begin
      AdjustColors(BevelOuter);
      Frame3D(Canvas,TmpRect,TopColor,BottomColor,BevelWidth);
    end;
  if BevelInner<>bvNone then
    begin
      AdjustColors(BevelInner);
      Frame3D(Canvas,TmpRect,TopColor,BottomColor,BevelWidth);
    end;
  with Canvas do
    begin
      Brush.Color := Color;
      FillRect(TmpRect);
    end;
  if FDigits<>nil then
    begin
      ValueString := Format('%*d',[FLength,FValue]);
      with TmpRect do
        begin
          Inc(Left,FMargin); Right := Left+FDigitWidth;
          Inc(Top,FMargin); Bottom := Top+FDigitHeight;
        end;
      for Digit:=1 to FLength do
        begin
          case ValueString[Digit] of
            '0'..'9':
              Canvas.CopyRect(TmpRect,FDigits.Canvas,
                              Rect((Ord(ValueString[Digit])-48)*FDigitWidth,0,(Ord(ValueString[Digit])-47)*FDigitWidth,FDigitHeight));
          else
              Canvas.CopyRect(TmpRect,FDigits.Canvas,
                              Rect(10*FDigitWidth,0,11*FDigitWidth,FDigitHeight));
          end;
        with TmpRect do
          begin
            Left := Right+FMargin; Right := Left+FDigitWidth;
          end;
      end;
    end;
end;

end.
